@@ -1,6 +1,6 @@
 ;;; sb-asahi.el --- shimbun backend for asahi.com -*- coding: iso-2022-7bit; -*-
 
-;; Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008
+;; Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
 ;; Yuuichi Teranishi <teranisi@gohome.org>
 
 ;; Author: TSUCHIYA Masatoshi <tsuchiya@namazu.org>,
@@ -36,8 +36,10 @@
 (eval-when-compile (require 'cl))
 
 (require 'shimbun)
+(require 'sb-multi)
 
-(luna-define-class shimbun-asahi (shimbun-japanese-newspaper shimbun) ())
+(luna-define-class shimbun-asahi (shimbun-japanese-newspaper shimbun-multi
+							     shimbun) ())
 
 (defvar shimbun-asahi-prefer-text-plain t
   "*Non-nil means prefer text/plain articles rather than html articles.")
@@ -217,7 +219,7 @@ Every `.' in NAME will be replaced with `/'."
        1 nil 2 6 3 4 5)
       ("digital" "$B%G%8%?%k(B" "%s/list.html"
        ,@(shimbun-asahi-make-regexp "digital/[^\"/]+"))
-      ("editorial" "$B<R@b(B" "paper/editorial.html"
+      ("editorial" "$B<R@b(B" "include/editorial_bno4.xml"
        ,(concat
 	 "<a" s1 "href=\"\\./"
 	 ;; 1. url
@@ -277,7 +279,7 @@ Every `.' in NAME will be replaced with `/'."
       ("sports.rugby" "$B%i%0%S!<(B")
       ("sports.usa" "$BJF%W%m%9%]!<%D(B")
       ("sports.winter" "$B%&%$%s%?!<%9%]!<%D(B")
-      ("tenjin" "$BE7@<?M8l(B" "paper/column.html"
+      ("tenjin" "$BE7@<?M8l(B" "include/column_bno4.xml"
        ,(concat
 	 "<a" s1 "href=\"\\./"
 	 ;; 1. url
@@ -292,7 +294,27 @@ Every `.' in NAME will be replaced with `/'."
 	 "\"")
        1 nil nil nil 2 3 4)
       ("travel" "$B%H%i%Y%k(B" "%s/news/"
-       ,@(shimbun-asahi-make-regexp "travel.news"))))
+       ,@(shimbun-asahi-make-regexp "travel.news"))
+      ("wakata" "$B<cED$5$s$-$\$&BZ:_5-(B" "%s/"
+       ,(concat
+	 "<a" s1 "href=\"/"
+	 ;; 1. url
+	 "\\(wakata\\(?:/[^\"/]+\\)*/"
+	 ;; 2. serial number
+	 "\\([a-z]*"
+	 ;; 3. year
+	 "\\(20[0-9][0-9]\\)"
+	 ;; 4. month
+	 "\\([01][0-9]\\)"
+	 ;; 5. day
+	 "\\([0-3][0-9]\\)"
+	 "[0-9]+\\)"
+	 "\\.html\\)"
+	 "\">" s0
+	 ;; 6. subject
+	 "\\([^\n<>]+\\)"
+	 s0 "\\(?:<img" s1 "[^>]+>" s0 "\\)*</a>")
+       1 nil 2 6 3 4 5)))
   "Alist of group names, their Japanese translations, index pages,
 regexps and numbers.  Where index pages and regexps may contain the
 \"%s\" token which is replaced with group names, numbers point to the
@@ -911,28 +933,36 @@ Each table is the same as the `cdr' of the element of
 `shimbun-asahi-group-table'.")
 
 (defvar shimbun-asahi-content-start
-  "<!--[\t\n ]*End of Headline[\t\n ]*-->\
+  "<div[\t\n ]+class=\"\
+\\(?:ThmbSet300Tb\\|ThmbSet256\\|Kansai-ThmbSet100\\|ThmbCol\\)\">\
+\\|<!--[\t\n ]*End of Headline[\t\n ]*-->\
 \\(?:[\t\n ]*<div[\t\n ]+[^<]+</div>[\t\n ]*\
 \\|[\t\n ]*<p[\t\n ]+[^<]+</p>[\t\n ]*\\)?\
 \\|<!--[\t\n ]*Start of \\(Kiji\\|photo\\)[\t\n ]*-->\
 \\|<!--[\t\n ]*FJZONE START NAME=\"HONBUN\"[\t\n ]*-->")
 
 (defvar shimbun-asahi-content-end
-  "\\(?:[\t\n ]*<[^>]+>\\)*[\t\n ]*<!--[\t\n ]*Start of hatenab[\t\n ]*-->\
-\\|<!--[\t\n ]*\\(?:google_ad_section\\|[AD$B!z!y(B]+\\)\
+  "<dl[\t\n ]+class=\"PrInfo\">\
+\\|<!--[\t\n ]*google_ad_section_end\
+\\|<!-[^>]+[^>$B!z(B]$B$3$3$^$G(B[\t\n ]*-+>\
+\\|\\(?:[\t\n ]*<[^>]+>\\)*[\t\n ]*<!--[\t\n ]*Start of hatenab[\t\n ]*-->\
 \\|<!--[\t\n ]*End of Kiji[\t\n ]*-->\
 \\|<!--[\t\n ]*End of related link[\t\n ]*-->\
 \\|<!--[\t\n ]*FJZONE END NAME=\"HONBUN\"[\t\n ]*-->")
 
 (defvar shimbun-asahi-text-content-start
-  "<!--[\t\n ]*End of Headline[\t\n ]*-->\
+  "<div[\t\n ]+class=\"\\(?:ThmbSet256\\|Kansai-ThmbSet100\\|ThmbCol\\)\">\
+\\|<!--[\t\n ]*End of Headline[\t\n ]*-->\
 \\(?:[\t\n ]*<div[\t\n ]+[^<]+</div>[\t\n ]*\
 \\|[\t\n ]*<p[\t\n ]+[^<]+</p>[\t\n ]*\\)?\
 \\|<!--[\t\n ]*Start of Kiji[\t\n ]*-->\
 \\|<!--[\t\n ]*FJZONE START NAME=\"HONBUN\"[\t\n ]*-->")
 
 (defvar shimbun-asahi-text-content-end
-  "\\(?:[\t\n ]*<[^>]+>\\)*[\t\n ]*<!--[\t\n ]*Start of hatenab[\t\n ]*-->\
+  "<dl[\t\n ]+class=\"PrInfo\">\
+\\|<!--[\t\n ]*google_ad_section_end\
+\\|<!-[^>]+$B$3$3$^$G(B[\t\n ]*-+>\
+\\|\\(?:[\t\n ]*<[^>]+>\\)*[\t\n ]*<!--[\t\n ]*Start of hatenab[\t\n ]*-->\
 \\|<!--[\t\n ]*\\(?:google_ad_section\\|[AD$B!z!y(B]+\\)\
 \\|<!--[\t\n ]*End of Kiji[\t\n ]*-->\
 \\|<!--[\t\n ]*FJZONE END NAME=\"HONBUN\"[\t\n ]*-->")
@@ -1177,22 +1207,27 @@ It works for only the groups `editorial' and `tenjin'."
   (goto-char (point-min))
   (let ((basename (cdr (assoc group '(("editorial" . "editorial")
 				      ("tenjin" . "column")))))
-	year month day url)
+	year cmonth month day url)
     (when (and basename
 	       (re-search-forward
-		(concat
-		 ;; 1. year
-		 "\\(20[0-9][0-9]\\)" "$BG/(B"
-		 ;; 2. month
-		 "\\([01]?[0-9]\\)" "$B7n(B"
-		 ;; 3. day
-		 "\\([0-3]?[0-9]\\)" "$BF|(B"
-		 "$B!J(B.$BMKF|!KIU(B")
+		(eval-when-compile
+		  (concat
+		   "/\\(?:editorial\\|column\\)\\.html\"[^\n0-9]+"
+		   ;; 1. month
+		   "\\([01]?[0-9]\\)" "$B7n(B"
+		   ;; 2. day
+		   "\\([0-3]?[0-9]\\)" "$BF|IU(B"))
 		nil t))
-      (setq year (string-to-number (match-string 1))
-	    month (string-to-number (match-string 2))
-	    day (string-to-number (match-string 3))
-	    url (format "paper/%s%d%02d%02d.html" basename year month day))
+      (setq year (shimbun-decode-time nil 32400)
+	    cmonth (nth 4 year)
+	    year (nth 5 year)
+	    month (string-to-number (match-string 1))
+	    day (string-to-number (match-string 2)))
+      (cond ((and (= cmonth 1) (= month 12))
+	     (decf year))
+	    ((and (= cmonth 12) (= month 1))
+	     (incf year)))
+      (setq url (format "paper/%s%d%02d%02d.html" basename year month day))
       (list
        (shimbun-make-header
 	;; number
@@ -1212,6 +1247,29 @@ It works for only the groups `editorial' and `tenjin'."
 	;; xref
 	(shimbun-expand-url url shimbun-asahi-url))))))
 
+(defun shimbun-asahi-multi-next-url (shimbun header url)
+  (goto-char (point-min))
+  (when (and (re-search-forward "\
+<div[\t\n ]\\(?:[^\t\n >]+[\t\n ]+\\)*class=\"SeqNav forSplit\""
+				nil t)
+	     (shimbun-end-of-tag "div" t))
+    (let ((end (match-beginning 0)))
+      (prog1
+	  (when (re-search-backward "\
+<a[\t\n ]+href=\"\\([^\"]+\\)\"[^>]*>[\t\n ]*$B<!%Z!<%8(B[\t\n ]*</a>"
+				    end 'move)
+	    (goto-char end)
+	    (shimbun-expand-url (match-string 1) url))
+	(if (and (re-search-backward "[^\t\n >]\\([\t\n ]*<\\)" nil t)
+		 (re-search-forward "[\t\n ]*<[\t\n ]*[^/]" end t))
+	    (delete-region (match-beginning 0) end)
+	  (goto-char end))
+	(insert "<!-- End of Kiji -->\n")))))
+
+(luna-define-method shimbun-multi-next-url ((shimbun shimbun-asahi)
+					    header url)
+  (shimbun-asahi-multi-next-url shimbun header url))
+
 (defun shimbun-asahi-prepare-article (shimbun header)
   "Prepare an article.
 For the groups editorial and tenjin, it tries to fetch the article for
@@ -1250,29 +1308,29 @@ that day if it failed."
      ((string-equal group "editorial")
       (let ((url (shimbun-header-xref header))
 	    (retry 0)
-	    start end)
+	    start)
 	(while retry
 	  (if (progn
 		(while (and (re-search-forward "\
-<\\(h[0-9]+\\)[\t\n ]+\\(?:[^\t\n >]+[\t\n ]+\\)*class=\"topi\""
+<\\(h[0-9]+\\)[\t\n ]+\\(?:[^\t\n >]+[\t\n ]+\\)*class=\"bdr_btm_2px"
 					       nil t)
-			    (shimbun-end-of-tag (match-string 1))
-			    (progn
-			      (delete-region (match-end 2) (match-end 0))
-			      (if end
-				  (progn
-				    (goto-char end)
-				    (delete-region end (match-beginning 2))
-				    (insert "</p>\n&#012;\n"))
-				(unless start
-				  (setq start (match-beginning 0)))
-				(delete-region start (match-beginning 2)))
-			      (re-search-forward "\
-<div[\t\n ]+\\(?:[^\t\n >]+[\t\n ]+\\)*class=\"kiji\""
-						 nil t))
-			    (setq end (shimbun-end-of-tag "div"))))
-		end)
+			    (shimbun-end-of-tag (match-string 1) t)
+			    (if start
+				(progn
+				  (delete-region
+				   (goto-char (match-beginning 0))
+				   (match-beginning 1))
+				  (insert "\n&#012;\n")
+				  (forward-char 1))
+			      (setq start (match-beginning 1)))))
+		(and start
+		     (re-search-forward "[\t\n ]*\
+\\(?:<ul[\t\n ]+\\(?:[^\t\n >]+[\t\n ]+\\)*class=\"FollowLnk\
+\\|<li[\t\n ]+\\(?:[^\t\n >]+[\t\n ]+\\)*class=\"TopLnk\"\
+\\|<dl[\t\n ]+\\(?:[^\t\n >]+[\t\n ]+\\)*class=\"PrInfo\"\\)"
+					nil t)))
 	      (progn
+		(goto-char (match-beginning 0))
 		(setq retry nil)
 		(insert "\n<!-- End of Kiji -->")
 		(goto-char start)
@@ -1284,8 +1342,9 @@ that day if it failed."
 	    (erase-buffer)
 	    (if (= retry 1)
 		(setq retry nil)
-	      (setq retry 1
-		    url (shimbun-index-url shimbun))
+	      (setq url "http://www.asahi.com/paper/editorial.html"
+		    retry 1
+		    start nil)
 	      (shimbun-header-set-xref header url)
 	      (shimbun-fetch-url shimbun url)
 	      (goto-char (point-min)))))))
@@ -1312,19 +1371,22 @@ that day if it failed."
      ((string-equal group "rss"))
      ((string-equal group "tenjin")
       (let ((url (shimbun-header-xref header))
-	    (retry 0))
+	    (retry 0)
+	    start)
 	(while retry
-	  (if (and
-	       (re-search-forward
-		"<div[\t\n ]+\\(?:[^\t\n >]+[\t\n ]+\\)*class=\"kiji\""
-		nil t)
-	       (shimbun-end-of-tag "div"))
+	  (if (and (re-search-forward "\
+<p>20[0-9][0-9]$BG/(B[01]?[0-9]$B7n(B[0-3]?[0-9]$BF|!J(B[$BF|7n2P?eLZ6bEZ(B]$B!KIU(B</p>\
+\\(\\(?:[\t\n ]*$B0u:~(B\\)?[\t\n ]*<[^>]+>\\)+[\t\n ]*"
+				      nil t)
+		   (progn
+		     (setq start (match-end 0))
+		     (re-search-forward "[\t\n ]*</p>" nil t)))
 	      (save-restriction
-		(narrow-to-region (match-beginning 0) (match-end 0))
+		(narrow-to-region start (goto-char (match-beginning 0)))
 		(setq retry nil)
-		(insert "\n<!-- End of Kiji -->")
-		(goto-char (match-beginning 0))
-		(insert "<!-- Start of Kiji -->")
+		(insert "</p>\n<!-- End of Kiji -->")
+		(goto-char start)
+		(insert "<!-- Start of Kiji --><p>")
 		(when (string-match "/column\\.html\\'" url)
 		  (insert "\
 \n<p>($B;XDj$5$l$?(B&nbsp;url&nbsp;$B$,(B&nbsp;$B$^$@(B/$B$9$G$K(B&nbsp;$BL5$$$N$G!"(B\
@@ -1342,8 +1404,9 @@ that day if it failed."
 	    (erase-buffer)
 	    (if (= retry 1)
 		(setq retry nil)
-	      (setq retry 1
-		    url (shimbun-index-url shimbun))
+	      (setq url "http://www.asahi.com/paper/column.html"
+		    retry 1
+		    start nil)
 	      (shimbun-header-set-xref header url)
 	      (shimbun-fetch-url shimbun url)
 	      (goto-char (point-min)))))))
@@ -1439,55 +1502,10 @@ that day if it failed."
 $B9-9p=*$o$j(B\\(?:[\t\n ]*</p>[\t\n ]*\\|\\'\\)"
 				  nil t)
 	   (delete-region start (match-end 0)))))
-     ;; Remove table tags that surround image tags.
-     (goto-char (point-min))
-     (let (end start found images)
-       (while (re-search-forward "[\t\n ]*<table[\t\n ]+[^>]+>[\t\n ]*\
-\\(?:\\(?:<[^>]+>[\t\n ]*\\)*\
-<img[\t\n ]+[^>]+>[\t\n ]*\\(?:<[^>]+>[\t\n ]*\\)*[^<]+\\)+\
-\\(?:<[^>]+>[\t\n ]*\\)*</table>[\t\n ]*"
-				 nil t)
-	 (setq found nil
-	       images nil
-	       end (match-end 0))
-	 (goto-char (setq start (match-beginning 0)))
-	 (while (re-search-forward "\
-\\(<img[\t\n ]+[^>]+>\\)[\t\n ]*\\(?:<[^>]+>[\t\n ]*\\)*\\([^<]+\\)"
-		 end t)
-	   (skip-chars-backward "\t\n ")
-	   (when (> (point) (match-beginning 2))
-	     (setq found t))
-	   (push (concat (match-string 1) "<br>"
-			 (buffer-substring (match-beginning 2) (point)))
-		 images)))
-	 (when found
-	   (setq images (nreverse images))
-	   (delete-region start end)
-	   (insert "\n")
-	   (while images
-	     (insert (pop images))
-	     (insert (if images "<br><br>\n" "\n")))))
-     ;; Remove garbage after images.
-     (goto-char (point-min))
-     (while (re-search-forward "\\(<img[\t\n ]+[^>]+>\\)[\t\n $B!!(B]*\
-\\(\\(?:<![^>]+>\\|<br>\\)[\t\n $B!!(B]*\\)*<p>"
-			       nil t)
-       (replace-match "\\1\n<p>"))
      ;; Remove any other useless things.
      (shimbun-remove-tags "[\t\n ]*<form[\t\n ]+" "</form>[\t\n ]*")
      (shimbun-remove-tags "[\t\n ]*<noscript>" "</noscript>[\t\n ]*")
      (shimbun-remove-tags "[\t\n ]*<script[\t\n ]" "</script>[\t\n ]*")
-     (goto-char (point-min))
-     (while (re-search-forward "[\t\n ]*\
-\\(?:<div[\t\n ]+[^>]+>\\|</div>\\|<ul>[\t\n ]*</ul>\\)\
-\[\t\n ]*"
-			       nil t)
-       (delete-region (match-beginning 0) (match-end 0)))
-     (goto-char (point-min))
-     (while (re-search-forward "[\t\n ]*\\(?:<[^>]+>[\t\n ]*\\)+\
-$B%"%5%R!&%3%`%H%C%W(B[$B$X%X(B]\\(?:\\(?:[\t\n ]*<[^>]+>\\)+[\t\n ]*\\|[\t\n ]*\\'\\)"
-			       nil t)
-       (replace-match "\n"))
      ;; Remove trailing garbage.
      (goto-char (point-min))
      (when (and (not (string-match "$B$f$k$f$k%U%'%_%K%s(B" from))
@@ -1507,13 +1525,110 @@ that day if it failed."
 						   header)
   (shimbun-asahi-prepare-article shimbun header))
 
-(luna-define-method shimbun-clear-contents :around ((shimbun shimbun-asahi)
-						    header)
+(defun shimbun-asahi-clear-contents (shimbun header)
   (when (luna-call-next-method)
+    ;; Remove table tags that surround image tags.
+    (goto-char (point-min))
+    (let (end start found images)
+      (while (re-search-forward "[\t\n ]*<table[\t\n ]+[^>]+>[\t\n ]*\
+\\(?:\\(?:<[^>]+>[\t\n ]*\\)*\
+<img[\t\n ]+[^>]+>[\t\n ]*\\(?:<[^>]+>[\t\n ]*\\)*[^<]+\\)+\
+\\(?:<[^>]+>[\t\n ]*\\)*</table>[\t\n ]*"
+				nil t)
+	(setq found nil
+	      images nil
+	      end (match-end 0))
+	(goto-char (setq start (match-beginning 0)))
+	(while (re-search-forward "\
+\\(<img[\t\n ]+[^>]+>\\)[\t\n ]*\\(?:<[^>]+>[\t\n ]*\\)*\\([^<]+\\)"
+				  end t)
+	  (skip-chars-backward "\t\n ")
+	  (when (> (point) (match-beginning 2))
+	    (setq found t))
+	  (push (concat (match-string 1) "<br>"
+			(buffer-substring (match-beginning 2) (point)))
+		images)))
+      (when found
+	(setq images (nreverse images))
+	(delete-region start end)
+	(insert "\n")
+	(while images
+	  (insert (pop images))
+	  (insert (if images "<br><br>\n" "\n")))))
+    ;; Remove zoom buttons.
+    (goto-char (point-min))
+    (while (re-search-forward "[\t\n ]*<img\\(?:[\t\n ]+[^\t\n >]+\\)*\
+\[\t\n ]+class=\"ThmbZoomBtn\"[^>]*>[\t\n ]*"
+			      nil t)
+      (replace-match "\n"))
+    ;; Remove garbage before images.
+    (goto-char (point-min))
+    (while (re-search-forward
+	    "\\(?:<\\(?:p\\|span\\)>[\t\n ]*\\)+\\(<img[\t\n ]+\\)"
+	    nil t)
+      (replace-match "\\1"))
+    ;; Remove garbage after images.
+    (goto-char (point-min))
+    (while (re-search-forward "\\(<img[\t\n ]+[^>]+>\\)[\t\n $B!!(B]*\
+\\(\\(?:<![^>]+>\\|<br>\\)[\t\n $B!!(B]*\\)*<p>"
+			      nil t)
+      (replace-match "\\1\n<p>"))
+    ;; Add line breaks after images that captions or images follow.
+    (goto-char (point-min))
+    (while (re-search-forward
+	    "\\(<img[\t\n ]+[^>]+>\\(?:[\t\n ]*</[^>]+>\\)*\\)[\t\n ]*"
+	    nil t)
+      (when (or (save-match-data
+		  (looking-at "\\(?:<[^\t\n >]+>[\t\n ]*\\)*<img[\t\n ]"))
+		(not (eq (char-after) ?<)))
+	(replace-match "\\1<br>\n")))
+    ;; Add line breaks before images that follow captions.
+    (goto-char (point-min))
+    (while (re-search-forward
+	    "[\t\n ]*\\(\\(?:<[^/>][^>]*>[\t\n ]*\\)*<img[\t\n ]\\)"
+	    nil t)
+      (unless (memq (char-before (match-beginning 0)) '(nil ?>))
+	(replace-match "<br>\n\\1")))
+    ;; Remove any other useless things.
+    (goto-char (point-min))
+    (while (re-search-forward "[\t\n ]*\
+\\(?:<div[\t\n ]+[^>]+>\\|</div>\\|<ul>[\t\n ]*</ul>\\)\
+\[\t\n ]*"
+			      nil t)
+      (delete-region (match-beginning 0) (match-end 0)))
+    (goto-char (point-min))
+    (while (re-search-forward "[\t\n ]*\\(?:<[^>]+>[\t\n ]*\\)+\
+\\(?:$B%"%5%R!&%3%`(B\\|$B%K%e!<%9(B\\)$B%H%C%W(B[$B$X%X(B]\
+\\(?:\\(?:[\t\n ]*<[!/][^>]+>\\)+[\t\n ]*\\|[\t\n ]*\\'\\)"
+			      nil t)
+      (replace-match "\n"))
     (unless (shimbun-prefer-text-plain-internal shimbun)
       (shimbun-break-long-japanese-lines))
     t))
 
+(luna-define-method shimbun-clear-contents :around ((shimbun shimbun-asahi)
+						    header)
+  (shimbun-asahi-clear-contents shimbun header))
+
+(defun shimbun-asahi-multi-clear-contents (shimbun header
+						   has-previous-page
+						   has-next-page)
+  (when (luna-call-next-method)
+    (when has-previous-page
+      (goto-char (point-min))
+      (insert "&#012;\n")
+      (when (looking-at "[\t\n ]*<p>[\t\n ]*")
+	(delete-region (match-beginning 0) (match-end 0))))
+    t))
+
+(luna-define-method shimbun-multi-clear-contents :around ((shimbun
+							   shimbun-asahi)
+							  header
+							  has-previous-page
+							  has-next-page)
+  (shimbun-asahi-multi-clear-contents shimbun header
+				      has-previous-page has-next-page))
+
 (provide 'sb-asahi)
 
 ;;; sb-asahi.el ends here
