@@ -1,6 +1,6 @@
 ;;; sb-kantei.el --- shimbun backend for kantei mail magazine backnumber -*- coding: iso-2022-7bit; -*-
 
-;; Copyright (C) 2001, 2003, 2003, 2004, 2005, 2006, 2007
+;; Copyright (C) 2001, 2003, 2003, 2004, 2005, 2006, 2007, 2008
 ;; Yuuichi Teranishi <teranisi@gohome.org>
 
 ;; Author: Yuuichi Teranishi <teranisi@gohome.org>
@@ -31,10 +31,12 @@
 
 (luna-define-class shimbun-kantei (shimbun) ())
 
-(defvar shimbun-kantei-url "http://www.kantei.go.jp/")
+(defvar shimbun-kantei-url "http://www.mmz.kantei.go.jp/")
 
 (defvar shimbun-kantei-groups '("m-magazine-en"
 				"m-magazine-ja"
+				"m-magazine-en.fukuda"
+				"m-magazine-ja.fukuda"
 				"m-magazine-en.abe"
 				"m-magazine-ja.abe"
 				"m-magazine-en.koizumi"
@@ -47,10 +49,15 @@ is for the backward compatibility.")
 (defvar shimbun-kantei-x-face-alist
   ;; Don't change the order of the faces.  See the method function that
   ;; is applied to `shimbun-make-contents'.
-  '(("default" . "X-Face: R![ems6?kedF&(},`\";7nbUIT6Uyt2A9jSQ'\\$=;,\
-n.9<lIS+DFBTdMEJ$nh0[t)XU!.D*p\n kd~cuh0/nvCm;1~B;Ib!g^TC*OHm5(<Z%=A2\
-H_,kDt0E*HaI&^C%Wzb/EC_PF1f!jk7VHf=s*mqe91\n `H.J(Bq9(S'71?$O\\+=Kp\"\
-yNww;MOGO&N+tm<MbYT}Mlh4<hahJgCV`P/<&9Fm|FRmb>vM+PFYQB}O\n <Se")
+  '(("default" . "X-Face: #(b'i|jCr9M1k*o`B1YbD.C*%\\T3~.mUK@q?}o4.TC\
+*~*~fPaHg\\]V+Q2$3wu$=:[<k^Y<s\n X{VB1rZN[(X$(Cej@QV?FaoslWKi,fxp\"m\\\
+<Cb#4vo!!hDZI@9I8gAMMp6>HZ'C/&9e15i*4e>OV4`\n pdAVvpz`w<$QCu9'~:}|h`S\
+EZv\\U]f']V(QbE5'%u$InJltT4~|Ru\\vs~g!;y;1uY#8v<8|eGbb*i=\n a/RM<?`*?\
+5`AL1#G(9F50D}`>Y:'\"^)0:;L!2x8j|br~q/E=j.s!FBI-6xr")
+    ("\\.fukuda\\'" . "X-Face: R![ems6?kedF&(},`\";7nbUIT6Uyt2A9jSQ'\\\
+$=;,n.9<lIS+DFBTdMEJ$nh0[t)XU!.D*p\n kd~cuh0/nvCm;1~B;Ib!g^TC*OHm5(<Z\
+%=A2H_,kDt0E*HaI&^C%Wzb/EC_PF1f!jk7VHf=s*mqe91\n `H.J(Bq9(S'71?$O\\+=\
+Kp\"yNww;MOGO&N+tm<MbYT}Mlh4<hahJgCV`P/<&9Fm|FRmb>vM+PFYQB}O\n <Se")
     ("\\.abe\\'" . "X-Face: 2lqMN=orK#d]Xl-K5P`=ApJHMB3[faCtca;G(i=qL\
 ^3qh<kEoLHF\"L\"x/a:|xD>x=IKEqN%\n 3EL93@D{*BW-{GE88b7{d^m-%v9}=-7=^M\
 #$?zJm$]Yy07J^}:#V?9t_<{fhavZVZQ1^1=SLQf3X=<\n z|Af_njD},U!m}4V}$]L_7\
@@ -66,6 +73,10 @@ REbDs'H9$Iy#yM#*J2c'L},(m8K:8?$vTPC%D}YJ[bV#7xw|{\"DJ:_?`V1m_4^+;7+\n\
     (concat (shimbun-url-internal shimbun)
 	    (cond ((string-equal group "m-magazine-en")
 		   "foreign/m-magazine/backnumber/")
+		  ((string-equal group "m-magazine-en.fukuda")
+		   "foreign/m-magazine/backnumber/fukuda.html")
+		  ((string-equal group "m-magazine-ja.fukuda")
+		   "jp/m-magazine/backnumber/hukuda.html")
 		  ((string-equal group "m-magazine-en.abe")
 		   "foreign/m-magazine/backnumber/abe.html")
 		  ((string-equal group "m-magazine-ja.abe")
@@ -82,7 +93,11 @@ REbDs'H9$Iy#yM#*J2c'L},(m8K:8?$vTPC%D}YJ[bV#7xw|{\"DJ:_?`V1m_4^+;7+\n\
 (luna-define-method shimbun-from-address ((shimbun shimbun-kantei))
   (let ((group (shimbun-current-group-internal shimbun)))
     (cond ((string-equal group "m-magazine-en")
+	   "Taro Aso")
+	  ((string-equal group "m-magazine-en.fukuda")
 	   "Yasuo Fukuda")
+	  ((string-equal group "m-magazine-ja.fukuda")
+	   "$BJ!ED9/IW(B")
 	  ((string-equal group "m-magazine-en.abe")
 	   "Shinzo Abe")
 	  ((string-equal group "m-magazine-ja.abe")
@@ -94,7 +109,7 @@ REbDs'H9$Iy#yM#*J2c'L},(m8K:8?$vTPC%D}YJ[bV#7xw|{\"DJ:_?`V1m_4^+;7+\n\
 	  ((string-equal group "m-magazine") ;; Backward compatibility.
 	   "$B>.@t=c0lO:(B")
 	  (t
-	   "$BJ!ED9/IW(B"))))
+	   "$BKc@8B@O:(B"))))
 
 (luna-define-method shimbun-get-headers ((shimbun shimbun-kantei)
 					 &optional range)
@@ -198,7 +213,56 @@ go[\t\n ]+to[\t\n ]+top[\t\n ]+of[\t\n ]+the[\t\n ]+page[\t\n ]*</a>\
 			       nil t)
 	    (delete-region (match-beginning 0) (point-max))
 	    (insert "\n")
-	    (delete-region (point-min) start))))))
+	    (delete-region (point-min) start)))
+      (goto-char (point-min))
+      (if (and (re-search-forward "<!--$BAmM}869F(B-->[\t\n ]*" nil t)
+	       (progn
+		 (setq start (match-end 0))
+		 (re-search-forward "[\t\n ]*<!--/$BAmM}869F(B-->" nil t)))
+	  (progn
+	    (delete-region (match-beginning 0) (point-max))
+	    (insert "\n")
+	    (delete-region (point-min) start))
+	;; Remove style sheet.
+	(goto-char (point-min))
+	(when (and (re-search-forward "<style[\t\n ]+" nil t)
+		   (shimbun-end-of-tag "style" t))
+	  (replace-match "\n"))
+	;; Remove navigation button.
+	(goto-char (point-min))
+	(when (and (re-search-forward "<\\(td\\|span\\)\
+\\(?:[\t\n ]+[^\t\n >]+\\)*[\t\n ]+class=\"breadcrumbs\""
+				      nil t)
+		   (shimbun-end-of-tag (match-string 1) t))
+	  (replace-match "\n")))
+      ;; Remove table tags.
+      (goto-char (point-min))
+      (while (re-search-forward "\
+\[\t\n ]*</?table\\(?:[\t\n ]+[^>]+\\)?>[\t\n ]*"
+				nil t)
+	(replace-match "\n"))
+      ;; Shrink boundary lines.
+      (let ((limit (w3m-static-if (featurep 'xemacs)
+		       (when (device-on-window-system-p)
+			 (font-width (face-font 'default)))
+		     (when window-system
+		       (frame-char-width)))))
+	(when limit
+	  (setq limit (* limit (1- (window-width))))
+	  (goto-char (point-min))
+	  (while (re-search-forward
+		  "<img\\(?:[\t\n ]+[^\t\n >]+\\)*[\t\n ]+height=\"1\""
+		  nil t)
+	    (when (shimbun-end-of-tag)
+	      (goto-char (match-beginning 0))
+	      (if (re-search-forward "width=\"\\([0-9]+\\)\"" (match-end 0) t)
+		  (when (> (string-to-number (match-string 1)) limit)
+		    (replace-match (concat "width=\"" (number-to-string limit)
+					   "\"")))
+		(goto-char (match-end 0)))))))
+      ;; Zenkaku ASCII -> Hankaku
+      (unless (memq (shimbun-japanese-hankaku shimbun) '(header subject nil))
+	(shimbun-japanese-hankaku-buffer t)))))
 
 (luna-define-method shimbun-make-contents :around ((shimbun shimbun-kantei)
 						   header)
@@ -210,12 +274,15 @@ go[\t\n ]+to[\t\n ]+top[\t\n ]+of[\t\n ]+the[\t\n ]+page[\t\n ]*</a>\
 		 (let ((from (shimbun-header-from header t)))
 		   `(lambda (ignore)
 		      ,(cdr (nth
-			     (cond ((member from '("Shinzo Abe"
-						   "$B0BG\?8;0(B"))
+			     (cond ((member from '("Yasuo Fukuda"
+						   "$BJ!ED9/IW(B"))
 				    1)
+				   ((member from '("Shinzo Abe"
+						   "$B0BG\?8;0(B"))
+				    2)
 				   ((member from '("Junichiro Koizumi"
 						   "$B>.@t=c0lO:(B"))
-				    2)
+				    3)
 				   (t
 				    0))
 			     shimbun-kantei-x-face-alist)))))))
