@@ -1,8 +1,8 @@
 ;;; sb-rss.el --- shimbun backend for RSS (Rich Site Summary).
 
-;; Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008
+;; Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009
 ;; Koichiro Ohba <koichiro@meadowy.org>
-;; Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008
+;; Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009
 ;; NAKAJIMA Mikio <minakaji@osaka.email.ne.jp>
 
 ;; Author: Koichiro Ohba <koichiro@meadowy.org>
@@ -41,7 +41,19 @@
     (require 'xml)))
 (eval '(require 'xml))
 
-(luna-define-class shimbun-rss (shimbun) ())
+(eval-and-compile
+  (luna-define-class shimbun-rss (shimbun) (ignored-subject)))
+
+(luna-define-method initialize-instance :after ((shimbun shimbun-rss)
+						&rest init-args)
+  (shimbun-rss-initialize-ignored-subject shimbun))
+
+(defun shimbun-rss-initialize-ignored-subject (shimbun)
+  (luna-set-slot-value shimbun 'ignored-subject
+		       (symbol-value
+			(intern-soft (format "shimbun-%s-ignored-subject"
+					     (shimbun-server shimbun)))))
+  shimbun)
 
 (luna-define-generic shimbun-rss-process-date (shimbun-rss date)
   "Process DATE string and return proper Date string to show it in MUA.")
@@ -172,6 +184,7 @@ But clarify need ignored URL return nil.")
 			 (shimbun-index-url shimbun)
 			 (error-message-string err))
 		nil)))
+	(ignored-subject (luna-slot-value shimbun 'ignored-subject))
 	dc-ns rss-ns author hankaku headers)
     (when xml
       (setq dc-ns (shimbun-rss-get-namespace-prefix
@@ -202,21 +215,24 @@ But clarify need ignored URL return nil.")
 		(let* ((date (or (shimbun-rss-get-date shimbun url)
 				 (shimbun-rss-node-text dc-ns 'date item)
 				 (shimbun-rss-node-text rss-ns 'pubDate item)))
-		       (id (shimbun-rss-build-message-id shimbun url date)))
-		  (when (and id (or need-all-items
-				    (not (shimbun-search-id shimbun id))))
+		       (id (shimbun-rss-build-message-id shimbun url date))
+		       (subject (shimbun-rss-node-text rss-ns 'title item)))
+		  (when (and id
+			     (or need-all-items
+				 (not (shimbun-search-id shimbun id)))
+			     (if (and ignored-subject subject)
+				 (not (string-match ignored-subject subject))
+			       t))
 		    (push
 		     (shimbun-create-header
 		      0
 		      (if hankaku
 			  (with-current-buffer hankaku
-			    (insert (or (shimbun-rss-node-text rss-ns 'title
-							       item)
-					""))
+			    (insert (or subject ""))
 			    (shimbun-japanese-hankaku-region (point-min)
 							     (point-max))
 			    (prog1 (buffer-string) (erase-buffer)))
-			(shimbun-rss-node-text rss-ns 'title item))
+			subject)
 		      (or (shimbun-rss-node-text rss-ns 'author item)
 			  (shimbun-rss-node-text dc-ns 'creator item)
 			  (shimbun-rss-node-text dc-ns 'contributor item)
