@@ -1,6 +1,6 @@
 ;;; shimbun.el --- interfacing with web newspapers -*- coding: iso-2022-7bit; -*-
 
-;; Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008
+;; Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
 ;; Yuuichi Teranishi <teranisi@gohome.org>
 
 ;; Author: TSUCHIYA Masatoshi <tsuchiya@namazu.org>,
@@ -78,6 +78,7 @@
 (require 'eword-encode)
 (require 'luna)
 (require 'std11)
+(require 'w3m)
 
 (eval-and-compile
   (luna-define-class shimbun ()
@@ -185,6 +186,21 @@ of `shimbun-SERVER-retry-fetching' overrides this variable."
 			 :match (lambda (widget value) (natnump value))
 			 :value 1)))
 
+(defcustom shimbun-use-local nil
+  "Specifies if local files should be used (\"offline\" mode).
+This way, you can use an external script to retrieve the
+necessary HTML/XML files.  For an example, see
+`nnshimbun-generate-download-script'.  If a local file for an URL
+cannot be found, it will silently be retrieved as usual."
+  :group 'shimbun
+  :type 'boolean)
+
+(defcustom shimbun-local-path w3m-default-save-directory
+  "Directory where local shimbun files are stored.
+Default is the value of `w3m-default-save-directory'."
+  :group 'shimbun
+  :type 'directory)
+
 (defun shimbun-servers-list ()
   "Return a list of shimbun servers."
   (let (servers)
@@ -196,8 +212,7 @@ of `shimbun-SERVER-retry-fetching' overrides this variable."
 	       (not (member (setq file (match-string 1 file))
 			    '("fml" "glimpse" "lump" "mailarc"
 			      "mailman" "mhonarc" "text" "hash"
-			      "rss" "rss-hash" "atom" "atom-hash"
-			      "multi")))
+			      "rss" "atom" "multi")))
 	       (not (member file servers))
 	       (push file servers)))))
     (sort servers 'string-lessp)))
@@ -219,20 +234,43 @@ of `shimbun-SERVER-retry-fetching' overrides this variable."
   (shimbun-mua-shimbun-internal mua))
 
 ;;; emacs-w3m implementation of url retrieval and entity decoding.
-(require 'w3m)
 (defun shimbun-retrieve-url (url &optional no-cache no-decode
 				 referer url-coding-system)
   "Rertrieve URL contents and insert to current buffer.
 Return content-type of URL as string when retrieval succeeded.
 Non-ASCII characters `url' are escaped based on `url-coding-system'."
-  (let (type)
+  (let (type charset fname)
     (if (and url
-	     (setq type (w3m-retrieve
-			 (w3m-url-transfer-encode-string url url-coding-system)
-			 nil no-cache nil referer)))
+	     shimbun-use-local
+	     shimbun-local-path
+	     (file-regular-p
+	      (setq fname (concat (file-name-as-directory
+				   (expand-file-name shimbun-local-path))
+				  (substring (md5 url) 0 10)
+				  "_shimbun"))))
+	;; get local file contents
+	(progn
+	  (let ((coding-system-for-read 'no-conversion))
+	    (insert-file-contents fname))
+	  (when (re-search-forward "^$" nil t)
+	    (let ((pos (match-beginning 0)))
+	      (re-search-backward
+	       "^Content-Type: \\(.*?\\)\\(?:[ ;]+\\|$\\)\\(charset=\\(.*\\)\\)?"
+	       nil t)
+	      (setq type (match-string 1)
+		    charset (match-string 3))
+	      (delete-region (point-min) pos))))
+      ;; retrieve URL
+      (when url
+	(setq type (w3m-retrieve
+		    (w3m-url-transfer-encode-string url url-coding-system)
+		    nil no-cache nil referer))))
+    (if type
 	(progn
 	  (unless no-decode
-	    (w3m-decode-buffer url)
+	    (if charset
+		(w3m-decode-buffer url charset type)
+	      (w3m-decode-buffer url))
 	    (goto-char (point-min)))
 	  type)
       (unless no-decode
@@ -924,13 +962,35 @@ Return nil if all pages should be retrieved."
        (if (eq 'all ,range) nil
 	 ,range))))
 
+(defvar shimbun-use-refresh t
+  "Non-nil means honor the REFRESH attribute in META tags.
+Bind it to nil per shimbun if the refresh brings unwanted page.")
+
+;; FIXME: It seems better that `shimbun-fetch-url' provides the redirection
+;; support, whereas it is currently done by `shimbun-headers-1' for headers
+;; and `shimbun-article-1' for articles separately.  The reason doing so is
+;; that `shimbun-article-1' needs to replace urls in XREFs with the redirected
+;; ones.
+(defun shimbun-headers-1 (shimbun url)
+  "Run `shimbun-fetch-url' and refresh the contents if necessary."
+  (let (;; The default url used when it is not specified for refresh.
+	(w3m-current-url url)
+	(w3m-use-refresh shimbun-use-refresh))
+    (when (and (shimbun-fetch-url shimbun url 'reload)
+	       (progn
+		 (w3m-check-refresh-attribute)
+		 (and w3m-current-refresh
+		      (not (string-equal url (cdr w3m-current-refresh))))))
+      (erase-buffer)
+      (shimbun-headers-1 shimbun (cdr w3m-current-refresh)))))
+
 (luna-define-method shimbun-headers ((shimbun shimbun) &optional range)
   (shimbun-message shimbun (concat shimbun-checking-new-news-format "..."))
   (prog1
       (with-temp-buffer
 	(let ((w3m-verbose (if shimbun-verbose nil w3m-verbose))
 	      headers)
-	  (shimbun-fetch-url shimbun (shimbun-index-url shimbun) 'reload)
+	  (shimbun-headers-1 shimbun (shimbun-index-url shimbun))
 	  (setq headers (shimbun-get-headers shimbun range))
 	  (if (memq (shimbun-japanese-hankaku shimbun) '(body nil))
 	      headers
@@ -1011,12 +1071,13 @@ Return nil when articles are not expired."
 HEADER is a shimbun-header which is obtained by `shimbun-headers'.
 If OUTBUF is not specified, article is retrieved to the current buffer.")
 
+;; See also the FIXME comment around `shimbun-headers-1'.
 (defun shimbun-article-1 (shimbun header)
   "Run `shimbun-fetch-url' and refresh the contents if necessary."
   (let* ((url (shimbun-article-url shimbun header))
 	 ;; The default url used when it is not specified for refresh.
 	 (w3m-current-url url)
-	 (w3m-use-refresh t)
+	 (w3m-use-refresh shimbun-use-refresh)
 	 real)
     (when (shimbun-fetch-url shimbun url nil nil
 			     (shimbun-article-base-url shimbun header))
@@ -1480,9 +1541,9 @@ it considers the buffer has already been narrowed to an article."
 	  (narrow-to-region start (match-beginning 0)))))
     (goto-char (point-min))
     (while (re-search-forward
-	    "<p[\t\n ]+[^>]*>\\|</p>\\|\\([$B!"!#!K!W(B]+\\)\\(\\cj\\)"
+	    "<p[\t\n ]+[^>]*>\\|</p>\\|\\([$B!"!#!K!W(B]+\\)\\(\\cj\\)?"
 	    nil t)
-      (if (match-beginning 1)
+      (if (match-beginning 2)
 	  (replace-match "\\1\n\\2")
 	(unless (eolp)
 	  (insert "\n")))))
@@ -1532,7 +1593,7 @@ it considers the buffer has already been narrowed to an article."
       (while (re-search-forward "\
 \\(?:[$B#F#f(B][$B#I#i(B][$B#L#l(B][$B#E#e(B]\\|[$B#F#f(B][$B#T#t(B][$B#P#p(B]\
 \\|[$B#H#h(B][$B#T#t(B][$B#T#t(B][$B#P#p(B][$B#S#s(B]?\\|[$B#M#m(B][$B#A#a(B][$B#I#i(B][$B#L#l(B][$B#T#t(B][$B#O#o(B]\\)\
-$B!'(B\\cA+"
+$B!'(B\\cj+"
 				nil t)
 	(japanese-hankaku-region (match-beginning 0) (match-end 0) t))
       (goto-char start)
