@@ -1,6 +1,6 @@
 ;;; w3m-bookmark.el --- Functions to operate bookmark file of w3m
 
-;; Copyright (C) 2001, 2002, 2003, 2005, 2006, 2007
+;; Copyright (C) 2001, 2002, 2003, 2005, 2006, 2007, 2008, 2009
 ;; TSUCHIYA Masatoshi <tsuchiya@namazu.org>
 
 ;; Authors: Shun-ichi GOTO     <gotoh@taiyo.co.jp>,
@@ -143,14 +143,10 @@ Minor mode to edit bookmark.
 (add-hook 'w3m-display-functions 'w3m-bookmark-mode-setter)
 
 (defun w3m-bookmark-file-modtime ()
-  "Return the bookmark file modification time.
-The value is a list of the form (HIGH . LOW), like the time values
-that `visited-file-modtime' returns.  When the bookmark file does not
-exist, returns (0 . 0)."
-  (if (file-exists-p w3m-bookmark-file)
-      (let ((time (nth 5 (file-attributes w3m-bookmark-file))))
-	(cons (car time) (cadr time)))
-    (cons 0 0)))
+  "Return the modification time of the bookmark file `w3m-bookmark-file'.
+The value is a list of two time values `(HIGH LOW)' if the bookmark
+file exists, otherwise nil."
+  (nth 5 (file-attributes w3m-bookmark-file)))
 
 (defun w3m-bookmark-buffer (&optional no-verify-modtime)
   "Return the buffer reading `w3m-bookmark-file' current."
@@ -164,7 +160,7 @@ exist, returns (0 . 0)."
       (with-current-buffer (w3m-get-buffer-create " *w3m bookmark*")
 	(unless (and w3m-bookmark-buffer-file-name
 		     (or no-verify-modtime
-			 (equal (visited-file-modtime)
+			 (equal (w3m-visited-file-modtime)
 				(w3m-bookmark-file-modtime))))
 	  (when (file-readable-p w3m-bookmark-file)
 	    (buffer-disable-undo)
@@ -185,7 +181,7 @@ exist, returns (0 . 0)."
 	(current-buffer)))))
 
 (defun w3m-bookmark-verify-modtime ()
-  (unless (equal (visited-file-modtime)
+  (unless (equal (w3m-visited-file-modtime)
 		 (w3m-bookmark-file-modtime))
     (if (buffer-file-name)
 	(ask-user-about-supersession-threat w3m-bookmark-file)
@@ -364,7 +360,9 @@ With prefix, ask for a new url instead of the present one."
 (defun w3m-bookmark-view (&optional reload)
   "Display the bookmark."
   (interactive "P")
-  (w3m-goto-url "about://bookmark/" reload))
+  (if (file-exists-p w3m-bookmark-file)
+      (w3m-goto-url "about://bookmark/" reload)
+    (message "No bookmark file is available")))
 
 ;;;###autoload
 (defun w3m-bookmark-view-new-session (&optional reload)
@@ -372,7 +370,9 @@ With prefix, ask for a new url instead of the present one."
   (interactive "P")
   (if (not (eq major-mode 'w3m-mode))
       (message "This command can be used in w3m mode only")
-    (w3m-view-this-url-1 "about://bookmark/" reload 'new-session)))
+    (if (file-exists-p w3m-bookmark-file)
+	(w3m-view-this-url-1 "about://bookmark/" reload 'new-session)
+      (message "No bookmark file is available"))))
 
 ;;;###autoload
 (defun w3m-about-bookmark (&rest args)
@@ -565,7 +565,8 @@ Format as (list (\"Group name\" . (\"Entry URL\" . \"Entry name\")* )* )."
 		    (w3m-bookmark-file-modtime)))
 	w3m-bookmark-menu-items-pre
       (setq w3m-bookmark-menu-items-time (w3m-bookmark-file-modtime))
-      (let ((entries (w3m-bookmark-iterator)))
+      (let ((entries (when (file-exists-p w3m-bookmark-file)
+		       (w3m-bookmark-iterator))))
 	(setq w3m-bookmark-menu-items-pre
 	      (and entries
 		   (mapcar
