@@ -348,11 +348,12 @@ If no field in forward, return nil without moving."
 	(mapconcat (lambda (elem)
 		     (setq elem (cdr elem))
 		     (format "%s=%s"
-			     (w3m-url-encode-string (car elem) coding)
+			     (w3m-url-encode-string (car elem) coding t)
 			     (w3m-url-encode-string (if (stringp (cdr elem))
 							(cdr elem)
 						      "")
-						    coding)))
+						    coding
+						    t)))
 		   bufs "&")))))
 
 (defun w3m-form-resume (forms)
@@ -540,7 +541,7 @@ fid=\\([^/]+\\)/type=\\([^/]+\\)/name=\\([^/]*\\)/id=\\(.*\\)$"
     (save-excursion
       (goto-char (point-min))
       (let (form fid start end type name rows start-column end-column
-		 hseq abs-hseq buffer-read-only text id filename)
+		 hseq abs-hseq buffer-read-only text id filename readonly)
 	(while (w3m-form-goto-next-field)
 	  (setq fid (get-text-property (point) 'w3m-form-field-id))
 	  (setq filename (get-text-property (point) 'w3m-form-file-name))
@@ -558,6 +559,7 @@ fid=\\([^/]+\\)/type=\\([^/]+\\)/name=\\([^/]*\\)/id=\\(.*\\)$"
 	    (when (string= type "textarea")
 	      (setq rows (get-text-property (point) 'w3m-textarea-rows)
 		    hseq (get-text-property (point) 'w3m-form-hseq)
+		    readonly (get-text-property (point) 'w3m-form-readonly)
 		    abs-hseq (w3m-anchor-sequence))
 	      (setq start-column (- (current-column) 1))
 	      (goto-char (next-single-property-change (point)
@@ -589,7 +591,8 @@ fid=\\([^/]+\\)/type=\\([^/]+\\)/name=\\([^/]*\\)/id=\\(.*\\)$"
 		     w3m-anchor-sequence ,abs-hseq
 		     w3m-form-id ,id
 		     w3m-form-name ,name
-		     w3m-form-file-name ,filename)))
+		     w3m-form-file-name ,filename
+		     w3m-form-readonly ,readonly)))
 		(when (setq text (w3m-form-get form id))
 		  (w3m-form-textarea-replace hseq text))))))))))
 
@@ -695,6 +698,7 @@ If optional REUSE-FORMS is non-nil, reuse it as `w3m-current-form'."
 			       (rows :integer)		 ; textarea
 			       (top_mergin :integer)	 ; textarea
 			       (checked :bool) ; checkbox, radio
+			       (readonly :bool)
 			       no_effect       ; map
 			       name value)
 	  (incf id)
@@ -747,7 +751,7 @@ If optional REUSE-FORMS is non-nil, reuse it as `w3m-current-form'."
 				 (w3m-url-decode-string value
 							w3m-current-coding-system)
 				 w3m-output-coding-system))
-		(setq textareas (cons (list textareanumber form id name)
+		(setq textareas (cons (list textareanumber form id name readonly)
 				      textareas)))
 	      (when w3m-current-url
 		(setq filename (expand-file-name
@@ -773,7 +777,8 @@ If optional REUSE-FORMS is non-nil, reuse it as `w3m-current-form'."
 		 w3m-anchor-sequence ,abs-hseq
 		 w3m-form-id ,id
 		 w3m-form-name ,name
-		 w3m-form-file-name ,filename)))
+		 w3m-form-file-name ,filename
+		 w3m-form-readonly ,readonly)))
 	     ((string= type "select")
 	      (when (if (eq w3m-type 'w3mmee)
 			(when value
@@ -804,7 +809,8 @@ If optional REUSE-FORMS is non-nil, reuse it as `w3m-current-form'."
 					     (w3m-form-get ,form ,id)
 					     w3m-form-new-session
 					     w3m-form-download)
-		 w3m-anchor-sequence ,abs-hseq)))
+		 w3m-anchor-sequence ,abs-hseq
+		 w3m-form-readonly ,readonly)))
 	     ((string= type "checkbox")
 	      (let ((cvalue (w3m-form-get form id)))
 		(w3m-form-put form id name
@@ -865,7 +871,8 @@ If optional REUSE-FORMS is non-nil, reuse it as `w3m-current-form'."
 					     (w3m-form-get ,form ,id)
 					     w3m-form-new-session
 					     w3m-form-download)
-		 w3m-anchor-sequence ,abs-hseq)))))))))
+		 w3m-anchor-sequence ,abs-hseq
+		 w3m-form-readonly ,readonly)))))))))
     ;; Process <internal> tag.
     (when (search-forward "<internal>" nil t)
       (setq internal-start (match-beginning 0))
@@ -954,26 +961,32 @@ If optional REUSE-FORMS is non-nil, reuse it as `w3m-current-form'."
       (goto-char p))))
 
 (defun w3m-form-input (form id name type width maxlength value)
-  (save-excursion
-    (let* ((fvalue (w3m-form-get form id))
-	   (input (read-from-minibuffer (concat (upcase type) ": ") fvalue))
-	   (coding (w3m-form-get-coding-system (w3m-form-charlst form))))
-      (when (with-temp-buffer
-	      (insert input)
-	      (w3m-form-coding-system-accept-region-p nil nil coding))
-	(w3m-form-put form id name input)
-	(w3m-form-replace input)))))
+  (let ((fvalue (w3m-form-get form id)))
+    (if (get-text-property (point) 'w3m-form-readonly)
+	(message "READONLY %s: %s" (upcase type) fvalue)
+      (save-excursion
+	(let ((input (save-excursion
+		       (read-from-minibuffer (concat (upcase type) ": ") fvalue)))
+	      (coding (w3m-form-get-coding-system (w3m-form-charlst form))))
+	  (when (with-temp-buffer
+		  (insert input)
+		  (w3m-form-coding-system-accept-region-p nil nil coding))
+	    (w3m-form-put form id name input)
+	    (w3m-form-replace input)))))))
 
 (defun w3m-form-input-password (form id name)
-  (let* ((fvalue (w3m-form-get form id))
-	 (input (read-passwd (concat "PASSWORD"
-				     (if fvalue
-					 " (default is no change)")
-				     ": ")
-			     nil
-			     fvalue)))
-    (w3m-form-put form id name input)
-    (w3m-form-replace input 'invisible)))
+  (if (get-text-property (point) 'w3m-form-readonly)
+      (message "This input box is read-only.")
+    (let* ((fvalue (w3m-form-get form id))
+	   (input (save-excursion
+		    (read-passwd (concat "PASSWORD"
+					 (if fvalue
+					     " (default is no change)")
+					 ": ")
+				 nil
+				 fvalue))))
+      (w3m-form-put form id name input)
+      (w3m-form-replace input 'invisible))))
 
 (defun w3m-form-input-checkbox (form id name value)
   (let ((fvalue (w3m-form-get form id)))
@@ -1017,9 +1030,10 @@ If optional REUSE-FORMS is non-nil, reuse it as `w3m-current-form'."
   (w3m-form-replace "*"))
 
 (defun w3m-form-input-file (form id name value)
-  (let ((input (read-file-name "File name: "
-			       (or (cdr (w3m-form-get form id))
-				   "~/"))))
+  (let ((input (save-excursion
+		 (read-file-name "File name: "
+				 (or (cdr (w3m-form-get form id))
+				     "~/")))))
     (w3m-form-put form id name (cons 'file input))
     (w3m-form-replace input)))
 
@@ -1078,7 +1092,7 @@ character."
     (goto-char p)))
 
 (defun w3m-form-textarea-info ()
-  "Return a list of (ID NAME LINE) for current text area."
+  "Return a list of (ID NAME LINE READONLY) for current text area."
   (let ((s (get-text-property (point) 'w3m-form-hseq))
 	(lines 1))
     (save-excursion
@@ -1086,7 +1100,8 @@ character."
 	(incf lines))
       (list (get-text-property (point) 'w3m-form-id)
 	    (get-text-property (point) 'w3m-form-name)
-	    lines))))
+	    lines
+	    (get-text-property (point) 'w3m-form-readonly)))))
 
 (defvar w3m-form-input-textarea-map nil)
 (unless w3m-form-input-textarea-map
@@ -1240,6 +1255,36 @@ textarea, or type \
 `\\<w3m-form-input-textarea-map>\\[w3m-form-input-textarea-exit]' to quit \
 textarea")))
 
+(eval-and-compile
+  (defalias 'w3m-same-window-p
+    (if (featurep 'xemacs)
+	(lambda (buffer-name)
+	  "Return non-nil if a buffer named BUFFER-NAME would be shown in the \"same\" window.
+This function returns non-nil if `display-buffer' or
+`pop-to-buffer' would show a buffer named BUFFER-NAME in the
+selected rather than \(as usual\) some other window.  See
+`same-window-buffer-names' and `same-window-regexps'."
+	  (cond
+	   ((not (stringp buffer-name)))
+	   ;; The elements of `same-window-buffer-names' can be buffer
+	   ;; names or cons cells whose cars are buffer names.
+	   ((and (boundp 'same-window-buffer-names)
+		 (member buffer-name same-window-buffer-names)))
+	   ((and (boundp 'same-window-buffer-names)
+		 (assoc buffer-name same-window-buffer-names)))
+	   ((and (boundp 'same-window-regexps)
+		 (save-match-data
+		   (catch 'found
+		     (dolist (regexp same-window-regexps)
+		       ;; The elements of `same-window-regexps' can be regexps
+		       ;; or cons cells whose cars are regexps.
+		       (when (or (and (stringp regexp)
+				      (string-match regexp buffer-name))
+				 (and (consp regexp) (stringp (car regexp))
+				      (string-match (car regexp) buffer-name)))
+			 (throw 'found t)))))))))
+      'same-window-p)))
+
 (defun w3m-form-input-textarea (form hseq)
   (let* ((info  (w3m-form-textarea-info))
 	 (value (w3m-form-get form (car info)))
@@ -1247,15 +1292,14 @@ textarea")))
 	 (wincfg (current-window-configuration))
 	 (w3mbuffer (current-buffer))
 	 (point (point))
-	 (size (min
-		(- (window-height cur-win)
-		   window-min-height 1)
-		(- (window-height cur-win)
-		   (max window-min-height
-			(1+ w3m-form-input-textarea-buffer-lines)))))
+	 (size (- (window-height cur-win)
+		  (1+ (max window-min-height
+			   w3m-form-input-textarea-buffer-lines))))
 	 (file (get-text-property (point) 'w3m-form-file-name))
 	 (coding (w3m-form-get-coding-system (w3m-form-charlst form)))
-	 (backup-p (w3m-form-use-textarea-backup-p))
+	 (readonly (nth 3 info))
+	 (backup-p (and (not readonly)
+			(w3m-form-use-textarea-backup-p)))
 	 buffer)
     (setq w3m-form-use-textarea-backup-p backup-p)
     (when backup-p
@@ -1287,11 +1331,17 @@ textarea")))
 		      (goto-char (abs (w3m-compare-strings
 				       before 0 (length before)
 				       value 0 (length value))))
-		      (y-or-n-p
-		       "The saved text for this form exists. Use it? ")))
+		      (condition-case nil
+			  (y-or-n-p
+			   "The saved text for this form exists. Use it? ")
+			(quit
+			 (kill-buffer buffer)
+			 (error "Abort textarea editing")))))
 	      (setq value before)))))
       (with-current-buffer buffer
 	(insert value)
+	(set-buffer-modified-p nil)
+	(when readonly (setq buffer-read-only t))
 	(goto-char (point-min))
 	(forward-line (1- (nth 2 info)))
 	(w3m-form-input-textarea-mode-setup w3mbuffer)
@@ -1307,13 +1357,20 @@ textarea")))
 	     (get-buffer-window (cdr buffer)))
 	;; same frame only
 	(select-window (get-buffer-window (cdr buffer)))
-      (condition-case nil
-	  (split-window cur-win (if (> size 0) size window-min-height))
-	(error
-	 (delete-other-windows)
-	 (split-window cur-win (- (window-height cur-win)
-				  w3m-form-input-textarea-buffer-lines))))
-      (select-window (next-window))
+      ;; Use the whole current window for the textarea when a user added
+      ;; the buffer name "*w3m form textarea*" to `same-window-buffer-names'
+      ;; (that is available only in Emacs).
+      ;; cf. http://article.gmane.org/gmane.emacs.w3m/7797
+      (unless (w3m-same-window-p (buffer-name (if (consp buffer)
+						  (cdr buffer)
+						buffer)))
+	(condition-case nil
+	    (split-window cur-win (if (> size 0) size window-min-height))
+	  (error
+	   (delete-other-windows)
+	   (split-window cur-win (- (window-height cur-win)
+				    w3m-form-input-textarea-buffer-lines))))
+	(select-window (next-window)))
       (let ((pop-up-windows nil))
 	(switch-to-buffer (if (consp buffer) (cdr buffer) buffer))))))
 
@@ -1455,6 +1512,8 @@ textarea")))
     'w3m-form-input-select-exit)
   (define-key w3m-form-input-select-keymap "q"
     'w3m-form-input-select-exit)
+  (define-key w3m-form-input-select-keymap "\C-g"
+    'w3m-form-input-select-exit)
   (define-key w3m-form-input-select-keymap "h" 'backward-char)
   (define-key w3m-form-input-select-keymap "j" 'next-line)
   (define-key w3m-form-input-select-keymap "k" 'previous-line)
@@ -1630,6 +1689,8 @@ textarea")))
     'w3m-form-input-map-exit)
   (define-key w3m-form-input-map-keymap "q"
     'w3m-form-input-map-exit)
+  (define-key w3m-form-input-map-keymap "\C-g"
+    'w3m-form-input-map-exit)
   (define-key w3m-form-input-map-keymap "h" 'backward-char)
   (define-key w3m-form-input-map-keymap "j" 'next-line)
   (define-key w3m-form-input-map-keymap "k" 'previous-line)
