@@ -1,6 +1,6 @@
 ;;; w3m-util.el --- Utility macros and functions for emacs-w3m
 
-;; Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007
+;; Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
 ;; TSUCHIYA Masatoshi <tsuchiya@namazu.org>
 
 ;; Authors: TSUCHIYA Masatoshi <tsuchiya@namazu.org>,
@@ -211,6 +211,15 @@ to the position where the property exists."
       `(get-text-property ,position 'w3m-image)
     `(w3m-get-text-property-around 'w3m-image)))
 
+(defmacro w3m-image-alt (&optional position)
+  "Return the value of the `w3m-image-alt' property at the given POSITION.
+NOTE: If POSITION is omitted, it searches for the property in one
+character before and behind the current position, and point will move
+to the position where the property exists."
+  (if position
+      `(get-text-property ,position 'w3m-image-alt)
+    `(w3m-get-text-property-around 'w3m-image-alt)))
+
 (defmacro w3m-submit (&optional position)
   "Return the value of the `w3m-submit' property at the given POSITION.
 NOTE: If POSITION is omitted, it searches for the property in one
@@ -240,7 +249,19 @@ If POSITION is omitted, the current position is assumed."
 (put 'w3m-parse-attributes 'lisp-indent-function '1)
 (def-edebug-spec w3m-parse-attributes
   ((&rest &or (symbolp &optional symbolp) symbolp) body))
-(defmacro w3m-parse-attributes (attributes &rest form)
+(defmacro w3m-parse-attributes (attributes &rest forms)
+  "Extract ATTRIBUTES, KEYWORD=\"VALUE\" pairs, in a tag and run FORMS.
+ATTRIBUTES is a list of symbols that looks like `(KEYWORD KEYWORD...)'.
+A symbol KEYWORD, that will express a value extracted from a tag, can
+be used as a Lisp variable within FORMS.  The point has to be within
+a tag initially, and only attributes that follow the point will be
+extracted.
+
+The value of KEYWORD is a string by default, or is nil if the KEYWORD
+is not found in a tag.  KEYWORD can be `(KEYWORD TYPE)', where TYPE is
+one of `:case-ignore', `:integer', `:bool', and `:decode-entity'.
+Those types mean converting the value into a lower-case string,
+an integer, a boolean (t or nil), and a decoded string respectively."
   `(let (,@(mapcar (lambda (attr)
 		     (if (listp attr)
 			 (car attr)
@@ -293,7 +314,7 @@ If POSITION is omitted, the current position is assumed."
        (skip-chars-forward " \t\r\f\n"))
      (skip-chars-forward "^>")
      (forward-char)
-     ,@form))
+     ,@forms))
 
 
 ;;; Working buffers:
@@ -673,9 +694,12 @@ objects will not be deleted:
 			    (if (eq w exception)
 				(setq flag nil)
 			      (set-buffer (window-buffer w))
-			      (setq flag (memq major-mode
-					       '(w3m-mode
-						 w3m-select-buffer-mode))))))
+			      (setq flag (or (memq major-mode
+						   '(w3m-mode
+						     w3m-select-buffer-mode
+						     w3m-session-select-mode))
+					     (string-match "\\` ?\\*w3m[ -]"
+							   (buffer-name)))))))
 			'no-minibuf)
 		       (set-buffer buffer)
 		       flag))
@@ -1019,10 +1043,9 @@ the region active."
 	 (setq zmacs-region-stays t))))
 
 (defmacro w3m-deactivate-region ()
-  "Deactivate the region.
-This macro does nothing in XEmacs, because the region is always
-deactivated after evaluating the current command."
-  (unless (featurep 'xemacs)
+  "Deactivate the region."
+  (if (featurep 'xemacs)
+      '(zmacs-deactivate-region)
     '(deactivate-mark)))
 
 (defmacro w3m-region-active-p ()
@@ -1031,19 +1054,20 @@ deactivated after evaluating the current command."
       (list 'region-active-p)
     (list 'and 'transient-mark-mode 'mark-active)))
 
-(cond
- ((fboundp 'replace-regexp-in-string)
-  (defun w3m-replace-in-string  (string regexp newtext &optional literal)
-    ;;(replace-regexp-in-string regexp newtext string nil literal)))
-    ;;
-    ;; Don't call the symbol function `replace-regexp-in-string' directly
-    ;; in order to silence the byte-compiler when an Emacs which doesn't
-    ;; provide it is used.  The following form generates exactly the same
-    ;; byte-code.
-    (funcall (symbol-function 'replace-regexp-in-string)
-	     regexp newtext string nil literal)))
- (t
-  (defalias 'w3m-replace-in-string 'replace-in-string)))
+(eval-and-compile
+  (cond
+   ((fboundp 'replace-regexp-in-string)
+    (defun w3m-replace-in-string  (string regexp newtext &optional literal)
+      ;;(replace-regexp-in-string regexp newtext string nil literal)))
+      ;;
+      ;; Don't call the symbol function `replace-regexp-in-string' directly
+      ;; in order to silence the byte-compiler when an Emacs which doesn't
+      ;; provide it is used.  The following form generates exactly the same
+      ;; byte-code.
+      (funcall (symbol-function 'replace-regexp-in-string)
+	       regexp newtext string nil literal)))
+   (t
+    (defalias 'w3m-replace-in-string 'replace-in-string))))
 
 (if (fboundp 'compare-strings)
     (defalias 'w3m-compare-strings 'compare-strings)
@@ -1081,6 +1105,36 @@ If SECONDS is omitted, it defaults to 0.5."
 		   buffer))
   (defalias 'w3m-force-window-update-later 'ignore))
 
+(if (fboundp 'read-number)
+    (defalias 'w3m-read-number 'read-number)
+  (defun w3m-read-number (prompt &optional default)
+    "Read a numeric value in the minibuffer, prompting with PROMPT.
+DEFAULT specifies a default value to return if the user just types RET.
+The value of DEFAULT is inserted into PROMPT."
+    (let ((n nil))
+      (when default
+	(setq prompt
+	      (if (string-match "\\(\\):[ \t]*\\'" prompt)
+		  (replace-match (format " (default %s)" default) t t prompt 1)
+		(w3m-replace-in-string prompt "[ \t]*\\'"
+				       (format " (default %s) " default)
+				       t))))
+      (while
+	  (progn
+	    (let ((str (read-from-minibuffer
+			prompt nil nil nil nil
+			(and default (number-to-string default)))))
+	      (condition-case nil
+		  (setq n (cond
+			   ((zerop (length str)) default)
+			   ((stringp str) (read str))))
+		(error nil)))
+	    (unless (numberp n)
+	      (message "Please enter a number.")
+	      (sit-for 1)
+	      t)))
+      n)))
+
 (defun w3m-make-menu-item (japan english)
   "Make menu item."
   (cond
@@ -1132,6 +1186,23 @@ If SECONDS is omitted, it defaults to 0.5."
     (set-buffer buffer)
     w3m-buffer-unseen))
 
+(defun w3m-visited-file-modtime ()
+  "Replacement of `visited-file-modtime'.
+It returns a list of two integers if the current buffer visits a file,
+otherwise returns the number 0.  In modern Emacsen, this function will
+get to be the alias to `visited-file-modtime'."
+  (let ((modtime (visited-file-modtime)))
+    (cond ((consp (cdr-safe modtime))
+	   (defalias 'w3m-visited-file-modtime 'visited-file-modtime)
+	   modtime)
+	  ((integerp (cdr-safe modtime))
+	   ;; XEmacs version returns `(0 . 0)' if no file is visited.
+	   (if (and (= (car modtime) 0) (= (cdr modtime) 0))
+	       0
+	     (list (car modtime) (cdr modtime))))
+	  (t
+	   modtime))))
+
 (provide 'w3m-util)
 
 ;;; w3m-util.el ends here
